@@ -16,6 +16,7 @@
 #include <kglobal.h>
 #include <kstatusbar.h>
 #include <kstdaction.h>
+#include <kstdgameaction.h>
 #include <kedittoolbar.h>
 #include <kkeydialog.h>
 #include <kdebug.h>
@@ -122,9 +123,8 @@ AbTop::~AbTop()
 
 void AbTop::setupActions()
 {
-  (void) new KAction( i18n("&New"), "new", 
-		      KStdAccel::key(KStdAccel::New), this,
-		      SLOT(newGame()), actionCollection(), "game_new");
+  KStdGameAction::gameNew( this, SLOT(newGame()), actionCollection() );
+  KStdGameAction::quit( this, SLOT(quit()), actionCollection() );
 
   (void) new KAction( i18n("&Stop Search"), "stop", Key_S, this,
 		      SLOT(stopSearch()), actionCollection(), "game_stop");
@@ -140,10 +140,6 @@ void AbTop::setupActions()
   (void) new KAction( i18n("&Hint"), "hint", Key_H, this,
 		      SLOT(suggestion()), actionCollection(), "game_hint");
 
-  (void) new KAction( i18n("&Quit"), "exit",
-		      KStdAccel::key(KStdAccel::Quit), this,
-		      SLOT(quit()), actionCollection(), "game_quit");
-
   KStdAction::copy( this, SLOT(copy()), actionCollection());
   KStdAction::paste( this, SLOT(paste()), actionCollection());
 
@@ -1248,3 +1244,4 @@ void AbTop::help()
 
 
 
+#include "AbTop.moc"
