@@ -3,25 +3,26 @@
  * Josef Weidendorfer, 10/97
  */
 
-#ifndef _SPION_H_
-#define _SPION_H_
+#ifndef _SPY_H_
+#define _SPY_H_
 
-#ifdef SPION
 
 #include <qlayout.h>
 #include "Board.h"
 
-clasee BoardWidget;
 
-class Spion: public QWidget
+class BoardWidget;
+class QLabel;
+
+class Spy: public QWidget
 {
   Q_OBJECT
 
 public:
-  Spion(Board&);
-  ~Spion() {};
+  Spy(Board&);
+  ~Spy() {};
 	
-  const int BoardCount = 5;
+  static const int BoardCount = 5;
 
   void clearActBoards();
 
@@ -40,6 +41,6 @@ private:
   QLabel *actLabel[BoardCount], *bestLabel[BoardCount];
 };
 
-#endif /* SPION */
+
 
 #endif /* _SPION_H_ */
