@@ -1,6 +1,6 @@
 /*
 
-    $Id: kmahjongg.h,v 1.18 2000/09/25 20:36:43 gebauer Exp $
+    $Id: kmahjongg.h,v 1.19 2001/03/08 17:40:44 mueller Exp $
 
     kmahjongg, the classic mahjongg game for KDE project
 
@@ -31,7 +31,7 @@
 class KMenuBar;
 class QLabel;
 
-#include <qfiledlg.h>
+#include <qfiledialog.h>
 
 #include <kapp.h>
 #include <kmessagebox.h>
